unit MMLabel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TMMLabel = class(TCustomLabel)
  private
    { Dclarations prives }
    FOnMouseEnter: TMouseMoveEvent;
    FOnMouseLeave: TMouseMoveEvent;
    FIsMouseIn : Boolean;

    FMouseOverColorActive : boolean;

    FMouseOverColor : TColor;
    FMainColor : TColor;

    FMouseOverFont : TFont;
    FMainFont: TFont;

    procedure MainFontChanged(Sender: TObject);
    procedure MouseOverFontChanged(Sender: TObject);

    procedure CMMouseEnter(var message : TWMMouseMove); message CM_MOUSEENTER;
    procedure CMMouseLeave(var message : TWMMouseMove); message CM_MOUSELEAVE;

    procedure WMainColor(const Value: TColor);
    procedure WMainFont(Value: TFont);
    procedure WMouseOverFont(Value: TFont);
    function IsMainFontStored: Boolean;
    function IsMouseOverFontStored: Boolean;

  protected
    { Dclarations protges }
    procedure DoMouseEnter(var Message: TWMMouseMove);
    procedure DoMouseLeave(var Message: TWMMouseMove);

  public
    { Dclarations publiques }
    constructor Create(AOwner : TComponent); override;
		destructor destroy; override;
  published
    { Dclarations publies }
    property OnMouseEnter : TMouseMoveEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave : TMouseMoveEvent read FOnMouseLeave write FOnMouseLeave;

    property IsMouseOverColorActive : Boolean read FMouseOverColorActive write FMouseOverColorActive;

    property MouseOverColor : TColor read FMouseOverColor write FMouseOverColor;
    property MainColor : TColor read FMainColor write WMainColor;
    property MainFont : TFont read FMainFont write WMainFont stored IsMainFontStored;
    property MouseOverFont : TFont read FMouseOverFont write WMouseOverFont stored IsMouseOverFontStored;

    property Align;
    property Alignment;
    property Anchors;
    property AutoSize;
    property BiDiMode;
    property Caption;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FocusControl;
    property ParentBiDiMode;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowAccelChar;
    property ShowHint;
    property Transparent;
    property Layout;
    property Visible;
    property WordWrap;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

procedure Register;

implementation

{$R *.DCR}

procedure Register;
begin
  RegisterComponents('Perso', [TMMLabel]);
end;

{ TMMLabel }
//______________________________________________________________________________
procedure TMMLabel.CMMouseEnter(var message: TWMMouseMove);
begin
  DoMouseEnter(Message);
end;

procedure TMMLabel.CMMouseLeave(var message: TWMMouseMove);
begin
  DoMouseLeave(Message);
end;

procedure TMMLabel.DoMouseEnter(var Message: TWMMouseMove);
begin
	FIsMouseIn := True;
	if IsMouseOverColorActive then
  begin
  	Color := FMouseOverColor;
	  Font := FMouseOverFont;
  end;
  if Assigned(FOnMouseEnter) then FOnMouseEnter(Self,[],0,0);
end;

procedure TMMLabel.DoMouseLeave(var Message: TWMMouseMove);
begin
	FIsMouseIn := False;
	Color := FMainColor;
  Font := FMainFont;
  if Assigned(FOnMouseLeave) then FOnMouseLeave(Self,[],0,0);
end;
//______________________________________________________________________________
function TMMLabel.IsMouseOverFontStored: Boolean;
begin
  Result := not ParentFont and not DesktopFont;
end;

function TMMLabel.IsMainFontStored: Boolean;
begin
  Result := not ParentFont and not DesktopFont;
end;
//______________________________________________________________________________
procedure TMMLabel.WMainColor(const Value: TColor);
begin
  FMainColor := Value;
  Color := FMainColor;
end;

procedure TMMLabel.WMainFont(Value: TFont);
begin
  FMainFont.Assign(Value);
  Font := FMainFont;
end;
//______________________________________________________________________________
procedure TMMLabel.WMouseOverFont(Value: TFont);
begin
  FMouseOverFont.Assign(Value);
end;
//______________________________________________________________________________
constructor TMMLabel.Create(AOwner: TComponent);
begin
  inherited;
  FMainFont := TFont.Create;
  FMainFont.Color := clWindowText;
  FMainFont.OnChange := MainFontChanged;

  FMouseOverFont := TFont.Create;
  FMouseOverFont.Color := clBlue;
  FMouseOverFont.OnChange := MouseOverFontChanged;
  FMouseOverFont.Style := FMouseOverFont.Style + [fsUnderline];
   
  IsMouseOverColorActive := True;
  MainColor := clBtnFace;
  MouseOverColor := clBtnFace;
end;

destructor TMMLabel.destroy;
begin
  FMainFont.Free;
  FMouseOverFont.Free;
  inherited;
end;

procedure TMMLabel.MainFontChanged(Sender: TObject);
begin
	if not FIsMouseIn then
  	Font := FMainFont;
end;

procedure TMMLabel.MouseOverFontChanged(Sender: TObject);
begin
	if FIsMouseIn then
  	Font := FMouseOverFont;
end;

end.
